function slice = iCTF_TiltThick_1d(sinogram,angles,params,varargin)
%ICTF_TILTTHICK_1D   Tilt+Thick CTF Correction for 1D projections
%
%
% SYNOPSIS:
%  slice = ICTF_TILTTHICK_1D(sinogram,angles,params,type,norders,lambda);
% 
% PARAMETERS:
%  sinogram:   Input sinogram (x,a)
%  angles:     Angles corresponding to sinogram
%  params:     set of microscope and acquisition parameters
%  type:       'Wiener', 'Phase' 
%  norders:    order of the Fourier expansion of the filter
%  lambda:     Regularisation parameter for Wiener filter
% 
% OUTPUT:
%  slice: 
%
% Note: The convention used in [1] for the order (N) of the filter is 
% different than the one used here: N = norders*2+1
% 
% [1] L.M. Voortman et al., Ultram. 2012, In press, 10.1016/j.ultramic.2012.05.002.

% The Quantitative Imaging Group of the TU Delft has developed and is the 
% owner of the image processing routines distributed in this package, 
% hereafter called SOFTWARE.
% 
% The SOFTWARE is free for non-commercial use by students and staff in 
% universities or non-profit research institutes.
% 
% Redistribution of SOFTWARE or parts thereof in any form is not permitted.
% 
% This SOFTWARE is distributed in the hope that it will be useful, but 
% without any warranty; without even the implied warranty of 
% merchantability or fitness for a particular purpose.

stdargin = nargin - size(varargin,2);
if nargin == stdargin
   type = 'Phase';
   norders = 7;
   lambda = 0;
end
if nargin == stdargin + 1
   type = varargin{1};
   norders = 7;
   lambda = 0.1;
end
if nargin == stdargin + 2
   type = varargin{1};
   norders = varargin{2};
   lambda = 0.1;
end
if nargin >= stdargin + 3
   type = varargin{1};
   norders = varargin{2};
   lambda = varargin{3};
end

sinogram = dip_image(sinogram);
N = size(sinogram,1);
nangles = length(angles);

%forward fourier transform
sinogram = dip_fouriertransform(sinogram,'forward',[1 0]);

% create set of frequency points
qx = double(xx(N,1,'freq'));
qz = 1/2*params.lambda*qx.^2*N/params.fov;

% weight for points in freq dom.
weight = 2*pi*abs(qx)*1/N/length(angles);
weight(N/2+1) = pi*(1/N/2)^2/length(angles);

% 
p  = xx(N,1)/params.fov;
W0 = -pi*params.lambda*params.df_estimated*p.^2 + pi/2*params.C_s*params.lambda^3*p.^4;

% iterate over orders of expansion
Ak_filter = getAkFilter(norders,type,lambda);      
[norders Ak_filter] = truncateAkFilter(Ak_filter);

Sinogram_stitch = newim(2*N,nangles*norders,'complex');
q_stitch = zeros(2,2*N*nangles*norders);

for k = 1:norders
   order = 2*k-1;

   q = zeros(2,2*N*nangles);
   for t = 1:nangles;
       R = [cos(angles(t)) sin(angles(t));-sin(angles(t)) cos(angles(t))];
       
       q(:,(2*t-2)*N+1:(2*t-1)*N) = R*[qx; order*qz];
       q(:,(2*t-1)*N+1:2*t*N)     = R*[qx; -order*qz];
   end
   q_stitch(:,(k-1)*2*N*nangles+1:k*2*N*nangles) = q;
   Sinogram_stitch(:,(k-1)*nangles:k*nangles-1) = [-1/(1i*exp(1i*order*W0))*Ak_filter(k)*sinogram*weight 1/(1i*exp(-1i*order*W0))*Ak_filter(k)*sinogram*weight];
end
q_stitch = mod(q_stitch+1,2)-1;
q_stitch = dip_image(q_stitch);
Sinogram_stitch = reshape(Sinogram_stitch,2*N*nangles*norders);

slice = real(nufft_type1(Sinogram_stitch,im2array(q_stitch),[N N]));
