function volume = iCTF_TiltThick_2d(stack,angles,params,varargin)
%ICTF_TILTTHICK_2D   Tilt+Thick CTF  correction for 2D projections
%
%
% SYNOPSIS:
%  volume = ICTF_TILTTHICK_2D(stack,angles,params,Nz,type,norders,lambda);
% 
% PARAMETERS:
%  stack:     Input stack (x,angles,y)
%  angles:    Angles at which to compute the projections
%  params:    set of microscope and acquisition parameters
%  Nz:        Size of reconstruction volume in z direction
%  type:      'Wiener', 'Phase' 
%  norders:   order of the Fourier expansion of the filter
%  lambda:    Regularisation parameter for Wiener filter
% 
% OUTPUT:
%  volume: 
%
% Note: The convention used in [1] for the order (N) of the filter is 
% different than the one used here: N = norders*2+1
% 
% [1] L.M. Voortman et al., Ultram. 2012, In press, 10.1016/j.ultramic.2012.05.002.

% The Quantitative Imaging Group of the TU Delft has developed and is the 
% owner of the image processing routines distributed in this package, 
% hereafter called SOFTWARE.
% 
% The SOFTWARE is free for non-commercial use by students and staff in 
% universities or non-profit research institutes.
% 
% Redistribution of SOFTWARE or parts thereof in any form is not permitted.
% 
% This SOFTWARE is distributed in the hope that it will be useful, but 
% without any warranty; without even the implied warranty of 
% merchantability or fitness for a particular purpose.

stack = dip_image(stack);

if ndims(stack) ~= 3
    error('stack should be 3D');
end
if size(stack,2) ~= numel(angles)
    error('number of angles does not match size of stack');
end

Nx = size(stack,1);
Ny = size(stack,3);

stdargin = nargin - size(varargin,2);
if nargin == stdargin
   Nz = Nx;
   type = 'Phase';
   norders = 7;
   lambda = 0;
end
if nargin >= stdargin + 1
   if ~isempty(varargin{1})
       Nz = varargin{1}; 
   else
       Nz = Nx;
   end
end
if nargin == stdargin + 1
   type = 'Phase';
   norders = 7;
   lambda = 0.1;
end
if nargin == stdargin + 2
   type = varargin{2};
   norders = 7;
   lambda = 0.1;
end
if nargin == stdargin + 3
   type = varargin{2};
   norders = varargin{3};
   lambda = 0.1;
end
if nargin >= stdargin + 4
   type = varargin{2};
   norders = varargin{3};
   lambda = varargin{4};
end

% allocate output memory
volume = newim(Nx,Nz,Ny);
volume = complex(volume,volume);

qy = yy(1,Ny,'freq');

% FT in y direction
stack = dip_fouriertransform(stack,'forward',[0 0 1]);

% iterate over y
for k = 0:Ny-1
    sinogram = squeeze(stack(:,:,k));

    slice = iCTF_TiltThick_2d_sinogram(sinogram,qy(0,k),angles,params,type,norders,lambda);

    volume(:,:,k) = cut(slice,[Nx Nz]);
end

% IFT in y direction
volume = real(dip_fouriertransform(volume,'inverse',[0 0 1]));
