function slice = iCTF_TiltThick_realspace_1d(sinogram,angles,params,varargin)
%ICTF_TILTTHICK_REALSPACE_1D   Tilt+Thick CTF Correction in real-space for 1D projections
%
%
% SYNOPSIS:
%  slice = ICTF_TILTTHICK_REALSPACE_1D(sinogram,angles,params,type,norders,lambda);
% 
% PARAMETERS:
%  sinogram:   Input sinogram (x,a)
%  angles:     Angles corresponding to sinogram
%  params:     set of microscope and acquisition parameters
%  type:       'Wiener', 'Phase' 
%  norders:   order of the Fourier expansion of the filter
%  lambda:     Regularisation parameter for Wiener filter
% 
% OUTPUT:
%  slice: 
%
% Note: The convention used in [1] for the order (N) of the filter is 
% different than the one used here: N = norders*2+1
% 
% [1] L.M. Voortman et al., Ultram. 2012, In press, 10.1016/j.ultramic.2012.05.002.

% The Quantitative Imaging Group of the TU Delft has developed and is the 
% owner of the image processing routines distributed in this package, 
% hereafter called SOFTWARE.
% 
% The SOFTWARE is free for non-commercial use by students and staff in 
% universities or non-profit research institutes.
% 
% Redistribution of SOFTWARE or parts thereof in any form is not permitted.
% 
% This SOFTWARE is distributed in the hope that it will be useful, but 
% without any warranty; without even the implied warranty of 
% merchantability or fitness for a particular purpose.

stdargin = nargin - size(varargin,2);
if nargin == stdargin
   type = 'Phase';
   norders = Inf;
   lambda = 0;
end
if nargin == stdargin + 1
   type = varargin{1};
   norders = Inf;
   lambda = 0.1;
end
if nargin == stdargin + 2
   type = varargin{1};
   norders = varargin{2};
   lambda = 0.1;
end
if nargin >= stdargin + 3
   type = varargin{1};
   norders = varargin{2};
   lambda = varargin{3};
end

sinogram = dip_image(sinogram);
N = size(sinogram,1);
nangles = length(angles);

slice = newim(N,N);

%forward fourier transform
sinogram = dip_fouriertransform(sinogram,'forward',[1 0]);

% create set of frequency points
qx = double(xx(N,1,'freq'));
z  = yy(1,N,'freq')*params.fov;

% weight for points in freq dom.
weight = 2*pi*abs(qx)*1/N/length(angles);
weight(N/2+1) = pi*(1/N/2)^2/length(angles);

p2  = xx(N,1).^2./params.fov.^2;
W0 = -pi*params.lambda*(params.df_estimated-z)*p2 + pi/2*params.C_s*params.lambda^3*p2.^2;
filter = getFilter(W0,type,lambda,norders)*weight;

for t = 1:nangles;
    spread = real(dip_fouriertransform(sinogram(:,t-1)*filter,'inverse',[1 0]));
    
    slice = slice + cut(rotation(spread,angles(t)),[N N]);
end
