function sinogram = iCTF_Tilt_1d(sinogram,angles,params,varargin)
%ICTF_TILT_1D   Tilted CTF correction for 1D projection
%
%
% SYNOPSIS:
%  sinogram = ICTF_TILT_1D(sinogram,angles,params,type,norders,lambda);
% 
% PARAMETERS:
%  sinogram:   Input projections (xp,angles)
%  angles:     Angles corresponding to sinogram
%  params:     set of microscope and acquisition parameters
%  type:       'Wiener', 'Phase'
%  norders:    order of the Fourier expansion of the filter
%  lambda:     Regularisation parameter for Wiener filter
% 
% OUTPUT:
%  sinogram:   
%
% Note: The convention used in [1] for the order (N) of the filter is 
% different than the one used here: N = norders*2+1
% 
% [1] L.M. Voortman et al., Ultram. 2012, In press, 10.1016/j.ultramic.2012.05.002.

% The Quantitative Imaging Group of the TU Delft has developed and is the 
% owner of the image processing routines distributed in this package, 
% hereafter called SOFTWARE.
% 
% The SOFTWARE is free for non-commercial use by students and staff in 
% universities or non-profit research institutes.
% 
% Redistribution of SOFTWARE or parts thereof in any form is not permitted.
% 
% This SOFTWARE is distributed in the hope that it will be useful, but 
% without any warranty; without even the implied warranty of 
% merchantability or fitness for a particular purpose.

stdargin = nargin - size(varargin,2);
if nargin == stdargin
   type = 'Phase';
   norders = 7;
   lambda = 0;
end
if nargin == stdargin + 1
   type = varargin{1};
   norders = 7;
   lambda = 0.1;
end
if nargin == stdargin + 2
   type = varargin{1};
   norders = varargin{2};
   lambda = 0.1;
end
if nargin >= stdargin + 3
   type = varargin{1};
   norders = varargin{2};
   lambda = varargin{3};
end

sinogram = dip_image(sinogram);
N = size(sinogram,1);
nangles = length(angles);

%forward fourier transform
Sinogram = dip_fouriertransform(sinogram,'forward',[1 0]);

tangles = newim(1,nangles);
tangles(:) = tan(angles);

qx = xx(N,1,'freq');
qz = -1/2*params.lambda*qx.^2*N/params.fov*tangles;

p  = qx*N/params.fov;
W0 = -pi*params.lambda*params.df_estimated*p.^2 + pi/2*params.C_s*params.lambda^3*p.^4;

Ak_filter = getAkFilter(norders,type,lambda);      
[norders Ak_filter] = truncateAkFilter(Ak_filter);

Sinogram_stitch = newim(2*N*norders,nangles,'complex');
q_stitch = newim(2*N*norders,nangles);

for k = 1:norders
   q = newim(2*N,nangles);

   order = 2*k-1;
   q(0:N-1,:) = qx - order*qz;
   q(N:2*N-1,:) = qx + order*qz;

   q_stitch((k-1)*2*N:k*2*N-1,:) = q;
   Sinogram_stitch((k-1)*2*N:k*2*N-1,:) = Ak_filter(k)*[1i*exp(-1i*order*W0)*Sinogram -1i*exp(1i*order*W0)*Sinogram];
end
q_stitch = mod(q_stitch+1,2)-1;

% compute NUFFT
sinogram = real(nufft_type1(Sinogram_stitch,q_stitch,[N nangles],[1 0]));
sinogram = sinogram/(N*sqrt(2));