function sinogram = iCTF_Tilt_Tikhonov_1d(sinogram,angles,params,varargin)
%ICTF_TILT_TIKHONOV_1D   Tilted CTF correction using Tikhonov regularisation for 1D projection
%
%
% SYNOPSIS:
%  sinogram = ICTF_TILT_TIKHONOV_1D(sinogram,angles,params,lambda);
% 
% PARAMETERS:
%  sinogram:   Input projections (xp,angles)
%  angles:     Angles corresponding to sinogram
%  params:     set of microscope and acquisition parameters
%  lambda:     Regularisation parameter for Wiener filter
% 
% OUTPUT:
%  sinogram:   
%
% Note: The convention used in [1] for the order (N) of the filter is 
% different than the one used here: N = norders*2+1
% 
% [1] L.M. Voortman et al., Ultram. 2012, In press, 10.1016/j.ultramic.2012.05.002.

% The Quantitative Imaging Group of the TU Delft has developed and is the 
% owner of the image processing routines distributed in this package, 
% hereafter called SOFTWARE.
% 
% The SOFTWARE is free for non-commercial use by students and staff in 
% universities or non-profit research institutes.
% 
% Redistribution of SOFTWARE or parts thereof in any form is not permitted.
% 
% This SOFTWARE is distributed in the hope that it will be useful, but 
% without any warranty; without even the implied warranty of 
% merchantability or fitness for a particular purpose.

stdargin = nargin - size(varargin,2);
if nargin == stdargin
   lambda = 0.1;
end
if nargin >= stdargin + 1
   lambda = varargin{1};
end

sinogram = dip_image(sinogram);
N = size(sinogram,1);
nangles = length(angles);

Sinogram = double(dip_fouriertransform(sinogram,'forward',[1 0]));

r = N/params.fov*linspace(-1/2,1/2-1/N,N);
W0 = -pi*params.lambda*params.df_estimated*r.^2 + pi/2*params.C_s*params.lambda^3*r.^4;
FTmat = exp(-2*pi*1i*(-N/2:N/2-1)'*(-N/2:N/2-1)/N)/sqrt(N);
x = (-N/2:N/2-1)*params.fov/N;
A  = pi*params.lambda*(r.^2)'*x;

for t = 1:nangles;
    WmatPhase = sin(W0'*ones(1,N) + A*tan(angles(t)));
    WmatPhaseFTmat = WmatPhase.*FTmat;

    thisproj = Sinogram(t,:);
    
    TikA = real(WmatPhaseFTmat'*WmatPhaseFTmat);
    TikA = TikA + lambda^2*eye(N);
    Tikb = real((WmatPhaseFTmat')*thisproj');
    sinogram(:,t-1) = TikA\Tikb;
end